#include "swTVector.h"
#include <vector>
#include "glut.h"


//--------------------------------------------------------------------------------------------------
//

void drawTriangle(const swTVector3<double>& vtx1, const swTVector3<double>& vtx2, const swTVector3<double>& vtx3)
{
	//glBegin(GL_TRIANGLES);
	glBegin(GL_LINE_LOOP);
		glNormal3dv(&vtx1[0]);  glVertex3dv(&vtx1[0]);
		glNormal3dv(&vtx2[0]);  glVertex3dv(&vtx2[0]);
		glNormal3dv(&vtx3[0]);  glVertex3dv(&vtx3[0]);
	glEnd();
}

void subdivide(const swTVector3<double>& vtx1, const swTVector3<double>& vtx2, const swTVector3<double>& vtx3, long depth)
{
	if (depth == 0) {
		drawTriangle(vtx1, vtx2, vtx3);
		return;
	}

	swTVector3<double> vtx12(vtx1 + vtx2), vtx23(vtx2 + vtx3), vtx31(vtx3 + vtx1);

	vtx12.normalize();
	vtx23.normalize();
	vtx31.normalize();

	subdivide(vtx1, vtx12, vtx31, depth - 1);
	subdivide(vtx2, vtx23, vtx12, depth - 1);
	subdivide(vtx3, vtx31, vtx23, depth - 1);
	subdivide(vtx12, vtx23, vtx31, depth - 1);
}

void drawIcosahedron(long depth)
{
	double X = 0.525731112119133606;
	double Z = 0.850650808352039932;

	::std::vector<swTVector3<double> > idata;
	idata.push_back(swTVector3<double>(-X, 0, Z));
	idata.push_back(swTVector3<double>(X, 0, Z));
	idata.push_back(swTVector3<double>(-X, 0, -Z));
	idata.push_back(swTVector3<double>(X, 0, -Z));
	idata.push_back(swTVector3<double>(0, Z, X));
	idata.push_back(swTVector3<double>(0, Z, -X));
	idata.push_back(swTVector3<double>(0, -Z, X));
	idata.push_back(swTVector3<double>(0, -Z, -X));
	idata.push_back(swTVector3<double>(Z, X, 0));
	idata.push_back(swTVector3<double>(-Z, X, 0));
	idata.push_back(swTVector3<double>(Z, -X, 0));
	idata.push_back(swTVector3<double>(-Z, -X, 0));
	
	::std::vector<swTVector3<int> > index;
	index.push_back(swTVector3<int>(0, 4, 1));
	index.push_back(swTVector3<int>(0, 9, 4));
	index.push_back(swTVector3<int>(9, 5, 4));
	index.push_back(swTVector3<int>(4, 5, 8));
	index.push_back(swTVector3<int>(4, 8, 1));
	index.push_back(swTVector3<int>(8, 10, 1));
	index.push_back(swTVector3<int>(8, 3, 10));
	index.push_back(swTVector3<int>(5, 3, 8));
	index.push_back(swTVector3<int>(5, 2, 3));
	index.push_back(swTVector3<int>(2, 7, 3));
	index.push_back(swTVector3<int>(7, 10, 3));
	index.push_back(swTVector3<int>(7, 6, 10));
	index.push_back(swTVector3<int>(7, 11, 6));
	index.push_back(swTVector3<int>(11, 0, 6));
	index.push_back(swTVector3<int>(0, 1, 6));
	index.push_back(swTVector3<int>(6, 1, 10));
	index.push_back(swTVector3<int>(9, 0, 11));
	index.push_back(swTVector3<int>(9, 11, 2));
	index.push_back(swTVector3<int>(9, 2, 5));
	index.push_back(swTVector3<int>(7, 2, 11));
	
	for (int i = 19; i >= 0; --i)
		subdivide(idata[index[i].x()], idata[index[i].y()], idata[index[i].z()], depth);
}
/*
struct SSurfaceParam {
	double u, v;
};

void calcSurface(const SSurfaceParam& sparam, swTVector3<double>& vtx, swTVector3<double>& normal)
{
	//  ...
}

double calcCurvature(const SSurfaceParam& sparam)
{
	// ...
}

void drawTriangle(const SSurfaceParam& sparam1, const SSurfaceParam& sparam2, const SSurfaceParam& sparam3)
{
	swTVector3<double> vtx1, vtx2, vtx3;
	swTVector3<double> normal1, normal2, normal3;

	calcSurface(sparam1, vtx1, normal1);
	calcSurface(sparam2, vtx2, normal2);
	calcSurface(sparam3, vtx3, normal3);

	glBegin(GL_TRIANGLES);
		glNormal3dv(&normal1[0]);  glVertex3dv(&vtx1[0]);
		glNormal3dv(&normal2[0]);  glVertex3dv(&vtx2[0]);
		glNormal3dv(&normal3[0]);  glVertex3dv(&vtx3[0]);
	glEnd();
}

void subdivide(const SSurfaceParam& sparam1, const SSurfaceParam& sparam2, const SSurfaceParam& sparam3, double cutoff, long depth)
{
	if (depth >= MAX_DEPTH ||
		(calcCurvature(sparam1) < cutoff && calcCurvature(sparam1) < cutoff && calcCurvature(sparam1) < cutoff)) {
		drawTriangle(sparam1, sparam2, sparam3);
		return;
	}

	swTVector3<double> sparam12, sparam23, sparam31;
	sparam12.u = (sparam1.u + sparam2.u) * 0.5;
	sparam23.u = (sparam2.u + sparam3.u) * 0.5;
	sparam31.u = (sparam2.u + sparam1.u) * 0.5;
	sparam12.v = (sparam1.v + sparam2.v) * 0.5;
	sparam23.v = (sparam2.v + sparam3.v) * 0.5;
	sparam31.v = (sparam2.v + sparam1.v) * 0.5;

	subdivide(sparam1, sparam12, sparam31, cutoff, depth + 1);
	subdivide(sparam2, sparam23, sparam12, cutoff, depth + 1);
	subdivide(sparam3, sparam31, sparam23, cutoff, depth + 1);
	subdivide(sparam12, sparam23, sparam31, cutoff, depth + 1);
}
*/
void render(void)
{
	GLfloat matSpecular[] = { 0.2f, 0.2f, 0.2f, 1.0f };
	GLfloat matShininess[] = { 5.0f };

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glMaterialfv(GL_FRONT, GL_SPECULAR, matSpecular);
	glMaterialfv(GL_FRONT, GL_SHININESS, matShininess);

	glPushMatrix();
		glTranslatef(-3.0f, 0.0f, 0.0f);
		glRotatef(25.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(-30.0f, 0.0f, 1.0f, 0.0f);

		drawIcosahedron(0l);
	glPopMatrix();
	
	glPushMatrix();
		glTranslatef(0.0f, 0.0f, 0.0f);
		glRotatef(25.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(-30.0f, 0.0f, 1.0f, 0.0f);

		drawIcosahedron(1l);
	glPopMatrix();
	
	glPushMatrix();
		glTranslatef(3.0f, 0.0f, 0.0f);
		glRotatef(25.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(-30.0f, 0.0f, 1.0f, 0.0f);

		drawIcosahedron(2l);
	glPopMatrix();
	   
	glFlush();
}

void initialize(void) 
{
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glShadeModel(GL_FLAT);
	
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
	
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	
	GLfloat light_position[] = { 0.0, 0.0, 1.0, 0.0 };
	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
}

void reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei)w, (GLsizei)h);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(30.0, double(w) / double(h), 1.0, 20.0);
	
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0f, 0.0f, -5.0f);
}

int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize(800, 200); 
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Mesh Simplification");
	
	initialize();
	
	glutDisplayFunc(render); 
	glutReshapeFunc(reshape); 
	//glutIdleFunc(render);
	glutMainLoop();
	
	return 0;
}
