#include "CartSimulator.h"
#include <cstdarg>
#include <cstdio>
#include <cmath>
#include "glut.h"


//--------------------------------------------------------------------------------------------------
//

static double pi = 4.0 * atan(1.0);


//--------------------------------------------------------------------------------------------------
// gprintf

void gprintf(float x, float y, const char* fmt, ...)
{
	char buffer[200] = { '\0', };
	//  setup
	va_list argptr;
	
	//  analyze
	va_start(argptr, fmt);
	vsprintf(buffer, fmt, argptr);
	va_end(argptr);
	
	//  draw text
	::glRasterPos3f(x, y, 0.0f);
	
    char c, *s = buffer;
	while((c = *s++) != 0) glutBitmapCharacter(GLUT_BITMAP_HELVETICA_18, c);
}


//--------------------------------------------------------------------------------------------------
// CCartSimulator

//  CCartSimulator 
CCartSimulator::CCartSimulator(double initVelCart, double uniformAccelCart)
: initVelCart_(initVelCart), uniformAccelCart_(uniformAccelCart), maxElapsedTime_(-initVelCart_ / uniformAccelCart_),
  step_(0ul), maxStep_(1000ul),
  pinnedBar_(uniformAccelCart, maxElapsedTime_, maxStep_)
{
}

//  ٴ ׸
void CCartSimulator::drawFloor(double len)
{
	glColor3f(0.0f, 0.0f, 0.0f);
	glBegin(GL_LINES);
		glVertex3d(-len / 2.0, 0.0, 0.0);
		glVertex3d(len / 2.0, 0.0, 0.0);
	glEnd();
}

//  cart ׸
void CCartSimulator::drawCart(double width, double height, double pos)
{
	glTranslated(pos, 0.0, 0.0);

	glColor3f(0.0f, 1.0f, 0.0f);
	glBegin(GL_QUADS);
		glVertex3d(-width / 2.0, 0.0, 0.0);
		glVertex3d(width / 2.0, 0.0, 0.0);
		glVertex3d(width / 2.0, height, 0.0);
		glVertex3d(-width / 2.0, height, 0.0);
	glEnd();
}

//  bar ׸
void CCartSimulator::drawBar(double len, double cartPos, double cartHeight, double inclinedAngle)
{
	glTranslated(cartPos, cartHeight / 2.0, 0.0);
	glRotated(inclinedAngle * 180 / pi, 0.0, 0.0, 1.0);

	double barWidth = 0.05;
	glColor3f(0.0f, 0.0f, 1.0f);
	glBegin(GL_QUADS);
		glVertex3d(-barWidth / 2.0, 0.0, 0.0);
		glVertex3d(-barWidth / 2.0, -len, 0.0);
		glVertex3d(barWidth / 2.0, -len, 0.0);
		glVertex3d(barWidth / 2.0, 0.0, 0.0);
	glEnd();
}

//  ð t  bar simulationѴ
void CCartSimulator::simulate()
{
	if (step_ < 0 || step_ > maxStep_) step_ = 0ul;

	double cartWidth = 0.5, cartHeight = 0.3;
	double cartPos = getCartPos(step_);

	glPushMatrix();
	glLoadIdentity();
		drawFloor(100.0);
	glPopMatrix();

	glPushMatrix();
	glLoadIdentity();
		drawCart(cartWidth, cartHeight, cartPos);
	glPopMatrix();

	glPushMatrix();
	glLoadIdentity();
		drawBar(pinnedBar_.getLength(), cartPos, cartHeight, pinnedBar_.getInclinedAngle(step_));
	glPopMatrix();

	++step_;
}

//  car ġ ȯ
double CCartSimulator::getCartPos(unsigned long step) const
{
	//  ܻ ó
	if (step < 0 && step > maxStep_) return 0.0;

	double currTime = maxElapsedTime_ * step / maxStep_;

	//  [ 13-35]  
	return currTime * currTime * uniformAccelCart_ / 2.0 + currTime * initVelCart_;
}
